@@ -3,6 +3,7 @@
 ;; Author: Vegard Øye <vegard_oye at hotmail.com>
 ;; Maintainer: Vegard Øye <vegard_oye at hotmail.com>
 
+;; Version: 1.15.0
 
 ;;
 ;; This file is NOT part of GNU Emacs.
@@ -24,10 +25,10 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with Evil.  If not, see <http://www.gnu.org/licenses/>.
 
-(require 'evil-core)
-
 ;;; Code:
 
+(require 'evil-core)
+
 ;;; Normal state
 
 (evil-define-state normal
@@ -35,7 +36,6 @@
 AKA \"Command\" state."
   :tag " <N> "
   :enable (motion)
-  :exit-hook (evil-repeat-start-hook)
   (cond
    ((evil-normal-state-p)
     (overwrite-mode -1)
@@ -47,8 +47,7 @@ AKA \"Command\" state."
   "Reset command loop variables in Normal state.
 Also prevent point from reaching the end of the line.
 If the region is activated, enter Visual state."
-  (unless (or (evil-initializing-p)
-              (null this-command))
+  (unless (null this-command)
     (setq command (or command this-command))
     (when (evil-normal-state-p)
       (setq evil-this-type nil
@@ -139,47 +138,50 @@ commands opening a new line."
   (remove-hook 'pre-command-hook #'evil-insert-repeat-hook))
 (put 'evil-insert-repeat-hook 'permanent-local-hook t)
 
+(eval-when-compile
+  ;; TODO remove this once support for emacs 26 is dropped
+  (unless (fboundp 'combine-change-calls)
+    (defmacro combine-change-calls (_beg _end &rest body) `(progn ,@body))))
+(declare-function evil-execute-repeat-info "evil-repeat")
 (defun evil-cleanup-insert-state ()
-  "Called when Insert state is about to be exited.
+  "Called when Insert or Replace state is about to be exited.
 Handles the repeat-count of the insertion command."
   (when evil-insert-count
-    (dotimes (_ (1- evil-insert-count))
-      (when evil-insert-lines
-        (evil-insert-newline-below)
-        (when evil-auto-indent
-          (indent-according-to-mode)))
-      (when (fboundp 'evil-execute-repeat-info)
-        (evil-execute-repeat-info
-         (cdr evil-insert-repeat-info)))))
+    (let (pre-command-hook post-command-hook)
+      (dotimes (_ (1- evil-insert-count))
+        (when evil-insert-lines
+          (evil-insert-newline-below)
+          (when evil-auto-indent
+            (indent-according-to-mode)))
+        (evil-execute-repeat-info (cdr evil-insert-repeat-info)))))
   (when evil-insert-vcount
-    (let ((buffer-invisibility-spec buffer-invisibility-spec))
-      ;; make all lines hidden by hideshow temporarily visible
-      (when (listp buffer-invisibility-spec)
-        (setq buffer-invisibility-spec
-              (evil-filter-list
-               #'(lambda (x)
-                   (or (eq x 'hs)
-                       (eq (car-safe x) 'hs)))
-               buffer-invisibility-spec)))
-      (let ((line (nth 0 evil-insert-vcount))
-            (col (nth 1 evil-insert-vcount))
-            (vcount (nth 2 evil-insert-vcount)))
+    (let ((buffer-invisibility-spec
+           (if (listp buffer-invisibility-spec)
+               ;; make all lines hidden by hideshow temporarily visible
+               (cl-remove-if (lambda (x) (eq (or (car-safe x) x) 'hs))
+                             buffer-invisibility-spec)
+             buffer-invisibility-spec)))
+      (cl-destructuring-bind (line col vcount) evil-insert-vcount
         (save-excursion
-          (dotimes (v (1- vcount))
-            (goto-char (point-min))
-            (forward-line (+ line v))
-            (when (or (not evil-insert-skip-empty-lines)
-                      (not (integerp col))
-                      (save-excursion
-                        (evil-move-end-of-line)
-                        (>= (current-column) col)))
-              (if (integerp col)
-                  (move-to-column col t)
-                (funcall col))
-              (dotimes (_ (or evil-insert-count 1))
-                (when (fboundp 'evil-execute-repeat-info)
-                  (evil-execute-repeat-info
-                   (cdr evil-insert-repeat-info)))))))))))
+          (combine-change-calls ; For performance
+              (progn (goto-char (point-min))
+                     (line-beginning-position line))
+              (line-end-position (+ line vcount -1))
+            (let (pre-command-hook post-command-hook) ; For performance
+              (dotimes (v (1- vcount))
+                (goto-char (point-min))
+                (forward-line (+ line v))
+                (when (or (not evil-insert-skip-empty-lines)
+                          (not (integerp col))
+                          (save-excursion
+                            (evil-move-end-of-line)
+                            (>= (current-column) col)))
+                  (if (integerp col)
+                      (move-to-column col t)
+                    (funcall col))
+                  (dotimes (_ (or evil-insert-count 1))
+                    (evil-execute-repeat-info (cdr evil-insert-repeat-info)))))))))))
+  (and evil-want-fine-undo (evil-end-undo-step)))
 
 ;;; Visual state
 
@@ -247,6 +249,7 @@ the selection is enabled.
                 (if (and (evil-visual-state-p)
                          (eq evil-visual-selection ',selection))
                     'exit ,name) t))
+         (setq evil--region-from-mouse nil)
          (if (eq type 'exit)
              (evil-exit-visual-state)
            (setq type (or type ,name)
@@ -279,7 +282,7 @@ the selection is enabled.
   ;; refresh the :corner property
   (setq evil-visual-properties
         (plist-put evil-visual-properties :corner
-                   (evil-visual-block-corner 'upper-left))))
+                   (evil-visual-block-corner))))
 
 (evil-define-state visual
   "Visual state."
@@ -324,15 +327,25 @@ the selection is enabled.
 Expand the region to the selection unless COMMAND is a motion."
   (when (evil-visual-state-p)
     (setq command (or command this-command))
-    (when evil-visual-x-select-timer
-      (cancel-timer evil-visual-x-select-timer))
-    (unless (evil-get-command-property command :keep-visual)
-      (evil-visual-update-x-selection)
-      (evil-visual-expand-region
-       ;; exclude final newline from linewise selection
-       ;; unless the command has real need of it
-       (and (eq (evil-visual-type) 'line)
-            (evil-get-command-property command :exclude-newline))))))
+    (if (and evil-transient-mouse-selection
+             evil--region-from-mouse
+             (not (rassq command evil-visual-alist)))
+        (progn (setq evil--region-from-mouse nil)
+               (evil-exit-visual-state))
+      (when (and evil-transient-mouse-selection
+                 evil--region-from-mouse
+                 (rassq command evil-visual-alist))
+        (setq evil-visual-selection nil ;; maintain visual state
+              evil--region-from-mouse nil))
+      (when evil-visual-x-select-timer
+        (cancel-timer evil-visual-x-select-timer))
+      (unless (evil-get-command-property command :keep-visual)
+        (evil-visual-update-x-selection)
+        (evil-visual-expand-region
+         ;; exclude final newline from linewise selection
+         ;; unless the command has real need of it
+         (and (eq (evil-visual-type) 'line)
+              (evil-get-command-property command :exclude-newline)))))))
 
 (put 'evil-visual-pre-command 'permanent-local-hook t)
 
@@ -343,12 +356,13 @@ otherwise exit Visual state."
   (when (evil-visual-state-p)
     (setq command (or command this-command))
     (if (or quit-flag
-            (eq command #'keyboard-quit)
+            (memq command '(keyboard-quit keyboard-escape-quit))
             ;; Is `mark-active' nil for an unexpanded region?
             deactivate-mark
             (and (not evil-visual-region-expanded)
                  (not (region-active-p))
-                 (not (eq evil-visual-selection 'block))))
+                 (not (eq evil-visual-selection 'block))
+                 transient-mark-mode))
         (progn
           (evil-exit-visual-state)
           (evil-adjust-cursor))
@@ -378,17 +392,16 @@ otherwise exit Visual state."
 (defun evil-visual-activate-hook (&optional _command)
   "Enable Visual state if the region is activated."
   (unless (evil-visual-state-p)
-    (evil-delay nil
-        ;; the activation may only be momentary, so re-check
-        ;; in `post-command-hook' before entering Visual state
-        '(unless (or (evil-visual-state-p)
-                     (evil-insert-state-p)
-                     (evil-emacs-state-p))
-           (when (and (region-active-p)
-                      (not deactivate-mark))
-             (evil-visual-state)))
-      'post-command-hook nil t
-      "evil-activate-visual-state")))
+    (evil-with-delay nil
+        (post-command-hook nil t "evil-activate-visual-state")
+      ;; the activation may only be momentary, so re-check
+      ;; in `post-command-hook' before entering Visual state
+      (unless (or (evil-visual-state-p)
+                  (evil-insert-state-p)
+                  (evil-emacs-state-p))
+        (when (and (region-active-p)
+                   (not deactivate-mark))
+          (evil-visual-state))))))
 (put 'evil-visual-activate-hook 'permanent-local-hook t)
 
 (defun evil-visual-deactivate-hook (&optional command)
@@ -419,6 +432,10 @@ If LATER is non-nil, exit after the current command."
           (setq deactivate-mark t)
         (when evil-visual-region-expanded
           (evil-visual-contract-region))
+        (setq evil-this-register nil
+              evil-prev-visual-selection evil-visual-selection
+              evil-prev-visual-mark (copy-marker evil-visual-mark)
+              evil-prev-visual-point (copy-marker evil-visual-point))
         (evil-change-to-previous-state)))))
 
 (defun evil-visual-tag (&optional selection)
@@ -435,17 +452,13 @@ or `block'."
 SELECTION is a kind of selection as defined by
 `evil-define-visual-selection', such as `char', `line'
 or `block'."
-  (let (message)
-    (setq selection (or selection evil-visual-selection))
-    (when selection
-      (setq message
-            (symbol-value (intern (format "evil-visual-%s-message"
-                                          selection))))
+  (unless selection (setq selection evil-visual-selection))
+  (when selection
+    (let ((message (symbol-value (intern (format "evil-visual-%s-message"
+                                                 selection)))))
       (cond
-       ((functionp message)
-        (funcall message))
-       ((stringp message)
-        (evil-echo "%s" message))))))
+       ((functionp message) (funcall message))
+       ((stringp message) (evil-echo "%s" message))))))
 
 (defun evil-visual-select (beg end &optional type dir message)
   "Create a Visual selection of type TYPE from BEG to END.
@@ -486,14 +499,12 @@ mark and point, use `evil-visual-select' instead."
 If TYPE is given, also set the Visual type.
 If MESSAGE is given, display it in the echo area."
   (interactive)
-  (let* ((point (evil-normalize-position
-                 (or point (point))))
-         (mark (evil-normalize-position
-                (or mark
-                    (when (or (evil-visual-state-p)
-                              (region-active-p))
-                      (mark t))
-                    point))))
+  (let* ((point (or point (point)))
+         (mark (or mark
+                   (when (or (evil-visual-state-p)
+                             (region-active-p))
+                     (mark t))
+                   point)))
     (unless (evil-visual-state-p)
       (evil-visual-state))
     (evil-active-region 1)
@@ -632,10 +643,6 @@ Reuse overlays where possible to prevent flicker."
   (let* ((point (point))
          (overlays (or overlays 'evil-visual-block-overlays))
          (old (symbol-value overlays))
-         (eol-col (and (memq this-command '(next-line previous-line))
-                       (numberp temporary-goal-column)
-                       (1+ (min (round temporary-goal-column)
-                                (1- most-positive-fixnum)))))
          beg-col end-col new nlines overlay window-beg window-end)
     (save-excursion
       ;; calculate the rectangular region represented by BEG and END,
@@ -646,16 +653,13 @@ Reuse overlays where possible to prevent flicker."
       (when (>= beg-col end-col)
         (if (= beg-col end-col)
             (setq end-col (1+ end-col))
-          (evil-sort beg-col end-col))
-        (setq beg (save-excursion
-                    (goto-char beg)
-                    (evil-move-to-column beg-col))
-              end (save-excursion
-                    (goto-char end)
-                    (evil-move-to-column end-col 1))))
-      ;; update end column with eol-col (extension to eol).
-      (when (and eol-col (> eol-col end-col))
-        (setq end-col eol-col))
+          (evil-swap beg-col end-col))
+        (setq beg (progn (goto-char beg) (evil-move-to-column beg-col))
+              end (progn (goto-char end) (evil-move-to-column end-col 1))))
+      ;; maybe extend end column to EOL
+      (and (memq this-command '(next-line previous-line evil-use-register))
+           (eq temporary-goal-column most-positive-fixnum)
+           (setq end-col most-positive-fixnum))
       ;; force a redisplay so we can do reliable window
       ;; BEG/END calculations
       (sit-for 0)
@@ -759,10 +763,9 @@ the direction of the last selection."
 (defun evil-visual-type (&optional selection)
   "Return the type of the Visual selection.
 If SELECTION is specified, return the type of that instead."
-  (if (and (null selection) (evil-visual-state-p))
-      (or evil-this-type (evil-visual-type evil-visual-selection))
-    (setq selection (or selection evil-visual-selection))
-    (symbol-value (cdr-safe (assq selection evil-visual-alist)))))
+  (or (and (null selection) (evil-visual-state-p) evil-this-type)
+      (symbol-value (cdr (assq (or selection evil-visual-selection)
+                               evil-visual-alist)))))
 
 (defun evil-visual-goto-end ()
   "Go to the last line of the Visual selection.
@@ -788,7 +791,8 @@ Default to `evil-visual-make-region'."
   "Return a Visual selection for TYPE."
   (catch 'done
     (dolist (selection evil-visual-alist)
-      (when (eq (symbol-value (cdr selection)) type)
+      (when (memq (symbol-value (cdr selection))
+                  (list type (evil-visual-type type)))
         (throw 'done (car selection))))))
 
 (defun evil-visual-block-corner (&optional corner point mark)
@@ -805,37 +809,22 @@ the horizontal or vertical component of CORNER is used.
 CORNER defaults to `upper-left'."
   (let* ((point (or point (point)))
          (mark (or mark (mark t)))
-         (corner (symbol-name
-                  (or corner
-                      (and (overlayp evil-visual-overlay)
-                           (overlay-get evil-visual-overlay
-                                        :corner))
-                      'upper-left)))
+         (corner (or corner
+                     (when evil-visual-overlay
+                       (overlay-get evil-visual-overlay :corner))
+                     'upper-left))
          (point-col (evil-column point))
          (mark-col (evil-column mark))
-         horizontal vertical)
-    (cond
-     ((= point-col mark-col)
-      (setq horizontal
-            (or (and (string-match "left\\|right" corner)
-                     (match-string 0 corner))
-                "left")))
-     ((< point-col mark-col)
-      (setq horizontal "left"))
-     ((> point-col mark-col)
-      (setq horizontal "right")))
-    (cond
-     ((= (line-number-at-pos point)
-         (line-number-at-pos mark))
-      (setq vertical
-            (or (and (string-match "upper\\|lower" corner)
-                     (match-string 0 corner))
-                "upper")))
-     ((< point mark)
-      (setq vertical "upper"))
-     ((> point mark)
-      (setq vertical "lower")))
-    (intern (format "%s-%s" vertical horizontal))))
+         (upperp (if (save-excursion (goto-char (min point mark))
+                                     (search-forward "\n" (max point mark) t))
+                     (< point mark)
+                   (memq corner '(upper-left upper-right))))
+         (leftp (if (= point-col mark-col)
+                    (memq corner '(upper-left lower-left))
+                  (< point-col mark-col))))
+    (if upperp
+        (if leftp 'upper-left 'upper-right)
+      (if leftp 'lower-left 'lower-right))))
 
 ;;; Operator-Pending state
 
@@ -867,16 +856,22 @@ CORNER defaults to `upper-left'."
   :tag " <R> "
   :cursor hbar
   :message "-- REPLACE --"
+  :entry-hook (evil-start-track-last-insertion)
+  :exit-hook (evil-cleanup-insert-state evil-stop-track-last-insertion)
   :input-method t
   (cond
    ((evil-replace-state-p)
     (overwrite-mode 1)
     (add-hook 'pre-command-hook #'evil-replace-pre-command nil t)
+    (add-hook 'pre-command-hook #'evil-insert-repeat-hook)
     (unless (eq evil-want-fine-undo t)
       (evil-start-undo-step)))
    (t
     (overwrite-mode -1)
     (remove-hook 'pre-command-hook #'evil-replace-pre-command t)
+    (remove-hook 'pre-command-hook #'evil-insert-repeat-hook)
+    (setq evil-insert-repeat-info evil-repeat-info)
+    (evil-set-marker ?^ nil t)
     (unless (eq evil-want-fine-undo t)
       (evil-end-undo-step))
     (evil-move-cursor-back)))
@@ -885,24 +880,23 @@ CORNER defaults to `upper-left'."
 (defun evil-replace-pre-command ()
   "Remember the character under point."
   (when (evil-replace-state-p)
-    (unless (assq (point) evil-replace-alist)
-      (add-to-list 'evil-replace-alist
-                   (cons (point)
-                         (unless (eolp)
-                           (char-after)))))))
+    (let ((char (unless (eolp) (char-after)))
+          (prev (assq (point) evil-replace-alist)))
+      (if prev
+          (setcdr prev char)
+        (push (cons (point) char) evil-replace-alist)))))
 (put 'evil-replace-pre-command 'permanent-local-hook t)
 
 (defun evil-replace-backspace ()
   "Restore character under cursor."
   (interactive)
-  (let (char)
-    (backward-char)
-    (when (assq (point) evil-replace-alist)
-      (setq char (cdr (assq (point) evil-replace-alist)))
-      (save-excursion
-        (delete-char 1)
-        (when char
-          (insert char))))))
+  (backward-char)
+  (let* ((prev (assq (point) evil-replace-alist))
+         (char (cdr prev))
+         (this-command #'evil-replace-backspace))
+    (when prev
+      (delete-char 1)
+      (when char (save-excursion (insert char))))))
 
 (defun evil-update-replace-alist (opoint count chars-to-delete &optional offset)
   "Add CHARS-TO-DELETE chars to evil-replace-alist, starting at OPOINT.
